/**
 * LICENCIA LGPL:
 * 
 * Esta librera es Software Libre; Usted puede redistribuirla y/o modificarla
 * bajo los trminos de la GNU Lesser General Public License (LGPL) tal y como 
 * ha sido publicada por la Free Software Foundation; o bien la versin 2.1 de 
 * la Licencia, o (a su eleccin) cualquier versin posterior.
 * 
 * Esta librera se distribuye con la esperanza de que sea til, pero SIN 
 * NINGUNA GARANTA; tampoco las implcitas garantas de MERCANTILIDAD o 
 * ADECUACIN A UN PROPSITO PARTICULAR. Consulte la GNU Lesser General Public 
 * License (LGPL) para ms detalles
 * 
 * Usted debe recibir una copia de la GNU Lesser General Public License (LGPL) 
 * junto con esta librera; si no es as, escriba a la Free Software Foundation 
 * Inc. 51 Franklin Street, 5 Piso, Boston, MA 02110-1301, USA o consulte
 * <http://www.gnu.org/licenses/>.
 *
 * Copyright 2011 Agencia de Tecnologa y Certificacin Electrnica
 */
package es.accv.arangi.base.exception.signature;

/**
 * Se trata de realizar una firma longeva con un certificado revocado o caducado. 
 * La firma longeva incluye informacin de validacin, y sta no puede indicar que 
 * el certificado est revocado.
 * 
 * @author <a href="mailto:jgutierrez@accv.es">Jos M Gutirrez</a>
 */
public class InvalidCertificateException extends es.accv.arangi.base.exception.SignatureException {

	private static final long serialVersionUID = 1L;
	
	private int errorNumber; 

	public InvalidCertificateException(int errorNumber) {
		super();
		this.errorNumber = errorNumber;
	}

	public InvalidCertificateException(String message, int errorNumber, Throwable cause) {
		super(message, cause);
		this.errorNumber = errorNumber;
	}

	public InvalidCertificateException(String message, int errorNumber) {
		super(message);
		this.errorNumber = errorNumber;
	}

	public InvalidCertificateException(Throwable cause, int errorNumber) {
		super(cause);
		this.errorNumber = errorNumber;
	}

	/**
	 * Obtiene el nmero del error que ocasion la excepcin. Las constantes que 
	 * determinan el resultado de una validacin se encuentran en
	 * {@link es.accv.arangi.base.util.validation.ValidationResult ValidationResult}
	 * @return nmero del error
	 */
	public int getErrorNumber() {
		return errorNumber;
	}

}
